// 
// Copyright (C) 2008, NinjaTrader LLC <www.ninjatrader.com>.
// NinjaTrader reserves the right to modify or overwrite this NinjaScript component with each release.
//
#region Using declarations
using System;
using System.ComponentModel;
using System.Diagnostics;
using System.Drawing;
using System.Drawing.Drawing2D;
using System.Xml.Serialization;
using NinjaTrader.Cbi;
using NinjaTrader.Data;
using NinjaTrader.Gui.Chart;
#endregion

// This namespace holds all indicators and is required. Do not change it.
namespace NinjaTrader.Indicator
{
    /// <summary>
    /// T3Colored Moving Average
    /// </summary>
    [Description("T3Colored Moving Average")]
    public class T3Colored : Indicator
    {
        #region Variables

            private double 					vFactor = 0.7; // Default setting for VFactor
			private int 					tCount = 3;
			private int 					period = 14;
			private Color					upColor					= Color.DeepSkyBlue;
			private Color					downColor				= Color.OrangeRed;
			private int						plot0Width				= 2;
			private PlotStyle				plot0Style				= PlotStyle.Line;
			private DashStyle				dash0Style				= DashStyle.Solid;
			private System.Collections.ArrayList seriesCollection;

        #endregion

        /// <summary>
        /// This method is used to configure the indicator and is called once before any bar data is loaded.
        /// </summary>
        protected override void Initialize()
        {
            Add(new Plot(Color.FromKnownColor(KnownColor.Green), PlotStyle.Line, "T3Colored"));
            Overlay				= true;
 			PlotsConfigurable	= false;
       }

		/// <summary>
		/// </summary>
		protected override void OnStartUp()
		{
			Plots[0].Pen.Width 		= plot0Width;
			Plots[0].PlotStyle		= plot0Style;
			Plots[0].Pen.DashStyle 	= dash0Style;
		}
		
		/// <summary>
        /// Called on each bar update event (incoming tick)
        /// </summary>
        protected override void OnBarUpdate()
        {
			if (TCount == 1)
			{
				CalculateGD(Inputs[0], Values[0]);
				return;
			}
				
			if (seriesCollection == null)
			{
				seriesCollection = new System.Collections.ArrayList();
				for (int i = 0; i < TCount - 1; i++) 
					seriesCollection.Add(new DataSeries(this));
			}
			
			CalculateGD(Inputs[0], (DataSeries) seriesCollection[0]);
			
			for (int i = 0; i <= seriesCollection.Count - 2; i++) 
				CalculateGD((DataSeries) seriesCollection[i], (DataSeries) seriesCollection[i + 1]);
			
			CalculateGD((DataSeries) seriesCollection[seriesCollection.Count - 1], Values[0]);
			
			if (Rising(Values[0]))
				PlotColors[0][0] = upColor;
			else 
				PlotColors[0][0] = downColor;
				
         }
		
		private void CalculateGD(IDataSeries input, DataSeries output)
		{
			output.Set((EMA(input, Period)[0] * (1 + VFactor)) - (EMA(EMA(input, Period), Period)[0] * VFactor));
		}
		
        #region Properties
		[Description("Numbers of bars used for calculations")]
        [GridCategory("Parameters")]
        public int Period
        {
            get { return period; }
            set { period = Math.Max(1, value); }
        }
		
		[Description("The smooth count")]
        [GridCategory("Parameters")]
        public int TCount
        {
            get { return tCount; }
            set { tCount = Math.Max(1, value); }
        }

        [Description("VFactor")]
        [GridCategory("Parameters")]
        public double VFactor
        {
            get { return vFactor; }
            set { vFactor = Math.Max(0, value); }
        }
 		/// <summary>
		/// </summary>
		[XmlIgnore()]
		[Description("Select color for rising T3")]
		[Category("Plots")]
		[Gui.Design.DisplayName("T3 Rising")]
		public Color UpColor
		{
			get { return upColor; }
			set { upColor = value; }
		}
		
		// Serialize Color object
		[Browsable(false)]
		public string UpColorSerialize
		{
			get { return NinjaTrader.Gui.Design.SerializableColor.ToString(upColor); }
			set { upColor = NinjaTrader.Gui.Design.SerializableColor.FromString(value); }
		}
		
		/// <summary>
		/// </summary>
		[XmlIgnore()]
		[Description("Select color for falling T3")]
		[Category("Plots")]
		[Gui.Design.DisplayName("T3 Falling")]
		public Color DownColor
		{
			get { return downColor; }
			set { downColor = value; }
		}
		
		// Serialize Color object
		[Browsable(false)]
		public string DownColorSerialize
		{
			get { return NinjaTrader.Gui.Design.SerializableColor.ToString(downColor); }
			set { downColor = NinjaTrader.Gui.Design.SerializableColor.FromString(value); }
		}
		
 		/// <summary>
		/// </summary>
		[Description("Width for T3P Line.")]
		[Category("Plots")]
		[Gui.Design.DisplayNameAttribute("Line Width")]
		public int Plot0Width
		{
			get { return plot0Width; }
			set { plot0Width = Math.Max(1, value); }
		}

		/// <summary>
		/// </summary>
		[Description("DashStyle for T3 Line")]
		[Category("Plots")]
		[Gui.Design.DisplayNameAttribute("Plot Style")]
		public PlotStyle Plot0Style
		{
			get { return plot0Style; }
			set { plot0Style = value; }
		}
		
		/// <summary>
		/// </summary>
		[Description("DashStyle for T3 Line")]
		[Category("Plots")]
		[Gui.Design.DisplayNameAttribute("Dash Style")]
		public DashStyle Dash0Style
		{
			get { return dash0Style; }
			set { dash0Style = value; }
		}
		
      #endregion
    }
}

#region NinjaScript generated code. Neither change nor remove.
// This namespace holds all indicators and is required. Do not change it.
namespace NinjaTrader.Indicator
{
    public partial class Indicator : IndicatorBase
    {
        private T3Colored[] cacheT3Colored = null;

        private static T3Colored checkT3Colored = new T3Colored();

        /// <summary>
        /// T3Colored Moving Average
        /// </summary>
        /// <returns></returns>
        public T3Colored T3Colored(int period, int tCount, double vFactor)
        {
            return T3Colored(Input, period, tCount, vFactor);
        }

        /// <summary>
        /// T3Colored Moving Average
        /// </summary>
        /// <returns></returns>
        public T3Colored T3Colored(Data.IDataSeries input, int period, int tCount, double vFactor)
        {
            if (cacheT3Colored != null)
                for (int idx = 0; idx < cacheT3Colored.Length; idx++)
                    if (cacheT3Colored[idx].Period == period && cacheT3Colored[idx].TCount == tCount && Math.Abs(cacheT3Colored[idx].VFactor - vFactor) <= double.Epsilon && cacheT3Colored[idx].EqualsInput(input))
                        return cacheT3Colored[idx];

            lock (checkT3Colored)
            {
                checkT3Colored.Period = period;
                period = checkT3Colored.Period;
                checkT3Colored.TCount = tCount;
                tCount = checkT3Colored.TCount;
                checkT3Colored.VFactor = vFactor;
                vFactor = checkT3Colored.VFactor;

                if (cacheT3Colored != null)
                    for (int idx = 0; idx < cacheT3Colored.Length; idx++)
                        if (cacheT3Colored[idx].Period == period && cacheT3Colored[idx].TCount == tCount && Math.Abs(cacheT3Colored[idx].VFactor - vFactor) <= double.Epsilon && cacheT3Colored[idx].EqualsInput(input))
                            return cacheT3Colored[idx];

                T3Colored indicator = new T3Colored();
                indicator.BarsRequired = BarsRequired;
                indicator.CalculateOnBarClose = CalculateOnBarClose;
#if NT7
                indicator.ForceMaximumBarsLookBack256 = ForceMaximumBarsLookBack256;
                indicator.MaximumBarsLookBack = MaximumBarsLookBack;
#endif
                indicator.Input = input;
                indicator.Period = period;
                indicator.TCount = tCount;
                indicator.VFactor = vFactor;
                Indicators.Add(indicator);
                indicator.SetUp();

                T3Colored[] tmp = new T3Colored[cacheT3Colored == null ? 1 : cacheT3Colored.Length + 1];
                if (cacheT3Colored != null)
                    cacheT3Colored.CopyTo(tmp, 0);
                tmp[tmp.Length - 1] = indicator;
                cacheT3Colored = tmp;
                return indicator;
            }
        }
    }
}

// This namespace holds all market analyzer column definitions and is required. Do not change it.
namespace NinjaTrader.MarketAnalyzer
{
    public partial class Column : ColumnBase
    {
        /// <summary>
        /// T3Colored Moving Average
        /// </summary>
        /// <returns></returns>
        [Gui.Design.WizardCondition("Indicator")]
        public Indicator.T3Colored T3Colored(int period, int tCount, double vFactor)
        {
            return _indicator.T3Colored(Input, period, tCount, vFactor);
        }

        /// <summary>
        /// T3Colored Moving Average
        /// </summary>
        /// <returns></returns>
        public Indicator.T3Colored T3Colored(Data.IDataSeries input, int period, int tCount, double vFactor)
        {
            return _indicator.T3Colored(input, period, tCount, vFactor);
        }
    }
}

// This namespace holds all strategies and is required. Do not change it.
namespace NinjaTrader.Strategy
{
    public partial class Strategy : StrategyBase
    {
        /// <summary>
        /// T3Colored Moving Average
        /// </summary>
        /// <returns></returns>
        [Gui.Design.WizardCondition("Indicator")]
        public Indicator.T3Colored T3Colored(int period, int tCount, double vFactor)
        {
            return _indicator.T3Colored(Input, period, tCount, vFactor);
        }

        /// <summary>
        /// T3Colored Moving Average
        /// </summary>
        /// <returns></returns>
        public Indicator.T3Colored T3Colored(Data.IDataSeries input, int period, int tCount, double vFactor)
        {
            if (InInitialize && input == null)
                throw new ArgumentException("You only can access an indicator with the default input/bar series from within the 'Initialize()' method");

            return _indicator.T3Colored(input, period, tCount, vFactor);
        }
    }
}
#endregion
